import java.rmi.*;

// Client for reading the server time.
public class ClockClient
{
	// Local clock
	static SynchronizedClock clock = new SynchronizedClock();

	// Synchronize the local clock with the server clock.
	static void syncClock(RemoteClock serverClock) throws RemoteException
	{
		// ... Add your solution here ... 
	}

	// Checks whether the client and server clocks are synchronized. To do this,
	// it checks whether the time provided by the server is within the local
	// time interval between the call and return of the remote method. If not,
	// there is obviously an inconsistency, since according to the timestamp,
	// either the request or the response would have to have arrived before it
	// was sent.
	static void checkClockSkew(RemoteClock serverClock) throws RemoteException
	{
		for (int i=0; i<10; i++) {
			long t1 = clock.getTime();
			long t2 = serverClock.getTime();
			long t3 = clock.getTime();

			if (t2 < t1) {
				System.out.println("Clocks are not synchronized: Request transfer time = "
								   + (t2-t1) + " < 0!");
				break;
			}
			if (t3 < t2) {
				System.out.println("Clocks are not synchronized: Response transfer time = "
								   + (t3-t2) + " < 0!");
				break;
			}
		}
	}

	// Main program: Check if clocks are sufficiently synchronized, synchronize
	// clocks, and check again. Repeat the check after one minute.
	public static void main(String args[])
	{
		String host = (args.length >= 1) ? args[0] : "localhost";
		try	{  
			RemoteClock serverClock = (RemoteClock)Naming.lookup("rmi://" + host + "/ClockServer");

			checkClockSkew(serverClock);

			syncClock(serverClock);

			checkClockSkew(serverClock);
            System.out.println("Waiting for 60 seconds before checking clock skew again");
			Thread.sleep(60000);
			checkClockSkew(serverClock);
		}
		catch(Exception ex) {
			System.out.println("Exception = " + ex ) ;
		}
	}
}

